import unittest
import six

from generic_encoders import dill_encoder

class TestDillEncoder(unittest.TestCase):

    def test_dill_encoder_mixed_type(self):
        string = {'a': 123, 'b': [1,2,{'c': 3}]}
        self.assertEqual(
          string,
          dill_encoder.decode(
            dill_encoder.encode(string)))

    def test_dill_handles_modules(self):
        self.assertEqual(
          123,
          dill_encoder.decode(dill_encoder.encode(dill_encoder)).decode(dill_encoder.encode(123))
          )

    def test_dill_handles_modules(self):
        self.assertEqual(
          123,
          dill_encoder.decode(dill_encoder.encode(dill_encoder)).decode(dill_encoder.encode(123))
          )


if __name__ == '__main__':
    unittest.main()


from .encoder import Encoder


class NoOpEncoder(object):
  file_suffixes = []
  inputs = [object]
  outputs = object

  def encode(self, data):
    return data

  def decode(self, data):
    return data



import csv
import six
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO
from itertools import chain
try:
    from itertools import imap
except ImportError:
    # Python 3...
    imap=map

from datetime import date, datetime
from .encoder import Encoder


class CsvEncoder(Encoder):
  file_suffixes = ['csv']
  inputs = [list]
  outputs = str

  def __init__(self, write_header=True, fieldnames=None):
    self.fieldnames = fieldnames
    self.write_header = write_header

  def _encode(self, data):
    if self.fieldnames is None:
        fieldnames = set(chain.from_iterable(imap(lambda x: x.keys(), data)))
    else:
        fieldnames = self.fieldnames
    f = StringIO()
    writer = csv.DictWriter(f, fieldnames)
    writer.writeheader()
    for record in data:
        writer.writerow(record)
    return f.getvalue()

  def _decode(self, data):
    f = StringIO(data)
    reader = csv.DictReader(f, self.fieldnames)
    return [x for x in reader]


import unittest
import six

from generic_encoders import snappy_encoder

class TestSnappyEncoder(unittest.TestCase):

    def test_snappy_encoder_bytes(self):
        string = b"some string 123"
        self.assertEqual(string,
            snappy_encoder.decode(
                snappy_encoder.encode(string)
                ))

    def test_string_throws_exception_on_p3(self):
        string = "some string 123"
        if not six.PY2:
            with self.assertRaises(TypeError) as context:
                snappy_encoder.encode(string)
        else:
            snappy_encoder.encode(string)

    def test_throws_exception_when_encode_passed_bogus_type(self):
        string = 123
        with self.assertRaises(TypeError) as context:
            snappy_encoder.encode(string)


    def test_throws_exception_when_decode_passed_bogus_type(self):
        string = 123
        with self.assertRaises(TypeError) as context:
            snappy_encoder.decode(string)


if __name__ == '__main__':
    unittest.main()


import six
import types
try:
    from io import BytesIO
except ImportError:
    from io import BytesIO

from .encoder import Encoder

def _print_import_error():
  print("Unable to import fastavro dependencies please install generic-encoders[avro]")

class AvroEncoder(Encoder):
  file_suffixes = ['avro']
  inputs = [list, types.GeneratorType]
  outputs = six.binary_type

  def __init__(self, avro_schema = None, codec='null', sync_interval=16000):
    self.avro_schema = avro_schema
    if avro_schema:
      try:
          from fastavro import parse_schema
      except ImportError:
        _print_import_error()
        raise
      self.parsed_avro_schema = parse_schema(avro_schema)
    else:
      self.parsed_avro_schema = None

    self.codec = codec
    self.sync_interval = sync_interval

  def _encode(self, data):
    try:
        from fastavro import writer
    except ImportError:
        _print_import_error()
        raise
    out = BytesIO()
    writer(out, self.parsed_avro_schema, data, codec=self.codec, sync_interval=self.sync_interval)
    return out.getvalue()

  def _decode(self, data):
    try:
        from fastavro import reader
    except ImportError:
        _print_import_error()
        raise
    fo = BytesIO(data)
    for record in reader(fo):
      yield record



import simplejson as json
import six

from datetime import date, datetime
from .encoder import Encoder


class JsonEncoder(Encoder):
  file_suffixes = ['json']
  inputs = [object]
  outputs = six.string_types[0]

  def __init__(self, sort_keys=True, skip_errors=False, default=None):
    self.sort_keys = sort_keys
    self.default = default
    if self.default and skip_errors:
      raise ValueError("Only one of skip_errors and default may be passed.")
    if not self.default:
      if skip_errors:
        self.default = self._json_serial_force
      else:
        self.default = self._json_serial
    super(JsonEncoder, self)

  def _encode(self, data):
    return json.dumps(
      data,
      sort_keys=self.sort_keys,
      default=self.default
      )

  def _decode(self, data):
    return json.loads(data)

  def _json_serial(self, obj):
      if isinstance(obj, (datetime, date)):
          return obj.isoformat()
      raise TypeError("Type {} not serializable".format(type(obj)))

  def _json_serial_force(self, obj):
      if isinstance(obj, (datetime, date)):
          return obj.isoformat()
      return str(obj)



import unittest
import six
from datetime import datetime

from generic_encoders import json_encoder
from generic_encoders.encoders.json_encoder import JsonEncoder

class TestJsonEncoder(unittest.TestCase):

    def test_json_encoder_mixed_type(self):
        string = {'a': 123, 'b': [1,2,{'c': 3}]}
        self.assertEqual(
          string,
          json_encoder.decode(
            json_encoder.encode(string)))

    def test_json_encoder_handles_datetimes(self):
        now = datetime.utcnow()
        self.assertEqual(
          now.isoformat(),
          json_encoder.decode(json_encoder.encode(now)))

    def test_json_encoder_handles_dates(self):
        now = datetime.utcnow().date()
        self.assertEqual(
          now.isoformat(),
          json_encoder.decode(json_encoder.encode(now)))

    def test_json_encoder_handles_dates(self):
        now = datetime.utcnow().date()
        self.assertEqual(
          now.isoformat(),
          json_encoder.decode(json_encoder.encode(now)))

    def test_json_encoder_throws_exception_when_passed_bogus(self):
        encoder = JsonEncoder(skip_errors=False)
        with self.assertRaises(TypeError) as context:
            encoder.encode(six)

    def test_json_encoder_can_skip_errors(self):
        encoder = JsonEncoder(skip_errors=True)
        encoder.encode(six)


if __name__ == '__main__':
    unittest.main()


import six

from .encoder import Encoder

class UnknownFormatException(Exception): pass
class EncoderLinkError(Exception): pass

class ComposedEncoder(Encoder):
  encoder_cache = {}

  def validate_encoder_connections(self, encoder, next_encoder):
    if not any([issubclass(encoder.outputs, x) for x in next_encoder.inputs]):
      raise EncoderLinkError(
        "Cannot connect {} which outputs ({}) to {} which accepts ({})".format(
          encoder,
          encoder.outputs,
          next_encoder,
          next_encoder.inputs
          ))

  def __init__(self, *args, **kwargs):
    self.encoders = args
    self.include_header = kwargs.get('include_header', False)
    self.check_for_headers = kwargs.get('check_for_headers', True)
    self.file_suffixes = []
    self.inputs = self.encoders[0].inputs
    self.outputs = self.encoders[-1].outputs
    last_encoder = None
    for encoder in self.encoders:
      if last_encoder:
        self.validate_encoder_connections(last_encoder, encoder)
      last_encoder = encoder
      self.file_suffixes.append(encoder.file_suffixes[0])

    self.header_prefix = kwargs.get('header_prefix', 'gpak') + '[' 
    self.header = '{}{}]'.format(
      self.header_prefix,
      '.'.join(self.file_suffixes))

  def has_header(self, data):
    if isinstance(data, six.string_types):
      return data.startswith(self.header_prefix)
    elif isinstance(data, six.binary_type):
      return data.startswith(self.header_prefix.encode('utf8'))
    return False

  @property
  def name(self):
    return ".".join([x.name for x in self.encoders])

  def split_header(self, data):
    header = self.header_prefix
    i = data.index('[')
    j = data.index(']')
    return data[i+1:j], data[j+1:]

  def _encode(self, data):
    for encoder in self.encoders:
      data = encoder.encode(data)
    if self.include_header:
      return self.header + data
    else:
      return data

  @classmethod
  def build_encoder_from_header(self, header):
    if header not in self.encoder_cache:
      parts = header.split('.')
      encoders = []
      for part in parts:
        if part not in self.all_file_suffixes:
          raise UnknownFormatException(part)
        encoders.append(self.all_file_suffixes[part])
      self.encoder_cache[header] = ComposedEncoder(*encoders)
    return self.encoder_cache[header]

  def _decode(self, data):
    if self.check_for_headers and self.has_header(data):
      header, data = self.split_header(data)
      encoder = self.build_encoder_from_header(header)
      return encoder.decode(data)

    for encoder in self.encoders[::-1]:
      data = encoder.decode(data)
    return data



import unittest
import six

from generic_encoders import csv_encoder

class TestCsvEncoder(unittest.TestCase):

    def test_csv_encoder_basic_dicts(self):
        dicts = [{"a": "1"}, {"a": "2"}, {"a": "3"}]
        self.assertEqual(
          dicts,
          csv_encoder.decode(
            csv_encoder.encode(dicts)))

    def test_csv_encoder_numbers_are_decoded_as_strings(self):
        dicts = [{"a":  1}, {"a": 2.2}]
        self.assertEqual(
          [{"a":  "1"}, {"a": "2.2"}],
          csv_encoder.decode(
            csv_encoder.encode(dicts)))




if __name__ == '__main__':
    unittest.main()


import six

from .encoder import Encoder


def _print_import_error():
  print("Unable to import snappy please install generic-encoders[snappy] & snappy system packages.")

class SnappyEncoder(Encoder):
  file_suffixes = ['snappy']
  inputs = [six.binary_type]
  outputs = six.binary_type

  def __init__(self):
    super(SnappyEncoder, self)

  def _encode(self, data):
    try:
        import snappy
    except ImportError:
        _print_import_error()
        raise
    return snappy.compress(data)

  def _decode(self, data):
    try:
        import snappy
    except ImportError:
        _print_import_error()
        raise
    return snappy.decompress(data)


import unittest

from generic_encoders import base64_encoder, url_base64_encoder

class Testbase64_encoderEncoder(unittest.TestCase):

    def test_base64_encoder_encoder(self):
        string = b"some string 123"
        self.assertEqual(string,
            base64_encoder.decode(
                base64_encoder.encode(string)
                ))

    def test_base64_encoder_decode(self):
        string = b"Hello"
        self.assertEqual(string,
            base64_encoder.decode(
                b'SGVsbG8='
                ))

    def test_url_base64_encoder_encode_decode(self):
        string = b"Hello"
        self.assertEqual(string,
            url_base64_encoder.decode(url_base64_encoder.encode(string)))




if __name__ == '__main__':
    unittest.main()


import unittest
import six

from generic_encoders import msgpack_encoder, MsgPackEncoder

class TestMsgpackEncoder(unittest.TestCase):

    def test_msgpack_encoder_mixed_type(self):
        obj = {'a': 123, 'b': (1,2,{'c': 3})}
        self.assertEqual(
          obj,
          msgpack_encoder.decode(
            msgpack_encoder.encode(obj)))

    def test_msgpack_encoder_converts_lists_to_tuples_by_default(self):
        l = [1,2,3]
        self.assertEqual(
          (1,2,3),
          msgpack_encoder.decode(
            msgpack_encoder.encode(l)))

    def test_msgpack_encoder_use_list(self):
        l = [1,2,3]
        encoder = MsgPackEncoder(use_list=True)
        self.assertEqual(
          l,
          encoder.decode(
            encoder.encode(l)))

    def test_msgpack_accepts_utf8_strings(self):
        string = '\xd1\x80\xd1\x83\xcc\x81\xd1\x81\xd1\x81\xd0\xba\xd0\xb8\xd0\xb9'
        encoder = MsgPackEncoder()
        self.assertEqual(
          string,
          encoder.decode(
            encoder.encode(string)))

    def test_msgpack_accepts_binary_data(self):
        string = b'\xc3\x83'
        encoder = MsgPackEncoder(raw=True, use_bin_type=True)
        self.assertEqual(
          string,
          encoder.decode(
            encoder.encode(string)))


if __name__ == '__main__':
    unittest.main()


import dill
import six

from .encoder import Encoder

class DillEncoder(Encoder):
  file_suffixes = ['dill', 'pkl']
  inputs = [object]
  outputs = six.binary_type

  def __init__(self):
    super(DillEncoder, self)

  def _encode(self, data):
    return dill.dumps(data)

  def _decode(self, data):
    return dill.loads(data)



import base64
import six
from .encoder import Encoder

class Base64Encoder(Encoder):
  file_suffixes = ['b64', 'base64']
  inputs = [six.binary_type]
  outputs = six.binary_type
  def __init__(self):
    super(Base64Encoder, self)

  _encode = staticmethod(base64.b64encode)
  _decode = staticmethod(base64.b64decode)

class UrlBase64Encoder(Encoder):
  file_suffixes = ['b64', 'base64']
  inputs = [six.binary_type]
  outputs = six.binary_type
  def __init__(self):
    super(UrlBase64Encoder, self)

  _encode = staticmethod(base64.urlsafe_b64encode)
  _decode = staticmethod(base64.urlsafe_b64decode)



import unittest
import six

from generic_encoders import lz4_encoder

class TestLz4Encoder(unittest.TestCase):

    def test_lz4_encoders_bytes(self):
        string = b"some string 123"
        self.assertEqual(string,
            lz4_encoder.decode(
                lz4_encoder.encode(string)
                ))

    def test_throws_exception_when_encode_passed_bogus_type(self):
        string = 123
        with self.assertRaises(TypeError) as context:
            lz4_encoder.encode(string)

    def test_throws_exception_when_decode_passed_bogus_type(self):
        string = 123
        with self.assertRaises(TypeError) as context:
            lz4_encoder.decode(string)

    def test_lz4_decode(self):
        string = b"asdaaaaaaaaaaaaaaaaasdasd\n"
        self.assertEqual(string,
            lz4_encoder.decode(
                b'\x04"M\x18d@\xa7\x0e\x00\x00\x00Lasda\x01\x00`sdasd\n\x00\x00\x00\x00+6\x98\xaf'
                ))

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
import unittest

from generic_encoders import (
    text_utf8_encoder,
    text_ascii_encoder,
    text_latin_1_encoder
    )

class TestTextEncoder(unittest.TestCase):

    def test_ascii_encoder(self):
        self.assertEqual(
            'abc',
            text_ascii_encoder.decode(text_ascii_encoder.encode('abc'))
            )

    def test_utf8_encoder(self):
        self.assertEqual(
            u'русский язык',
            text_utf8_encoder.decode(text_utf8_encoder.encode(u'русский язык'))
            )



if __name__ == '__main__':
    unittest.main()



class Encoder(object):
  """
  Encoder is an abstract class which defines an interface
  for encoding data.

  Subclasses must provide a inputs class variable which lists acceptable
  input types.

  Subclasses must also provide an outputs class variable which specifies the
  output type produced.

  Subclasses may optionally provide a file_suffixes class variable which
  lists the possible suffixes files encoded via this encoder may be stored as.
  """

  file_suffixes = []
  all_file_suffixes = {}

  @classmethod
  def add_encoder(cls, encoder):
    for file_suffix in encoder.file_suffixes:
      cls.all_file_suffixes[file_suffix] = encoder

  def __init__(self):
    if not hasattr(self, 'inputs'):
      raise NotImplementedError('Encoders must set inputs')

    if not hasattr(self, 'outputs'):
      raise NotImplementedError('Encoders must set outputs')

  @property
  def inverted(self):
    return InvertedEncoder(self)

  def _encode(self, data):
    raise NotImplementedError('encode')

  def _decode(self, data):
    raise NotImplementedError('decode')

  def type_matches(self, data, inputs):
    for _type in inputs:
      if isinstance(data, _type):
        return True
    return False

  @property
  def name(self):
    return self.__class__.__name__

  def encode(self, data):
    if not self.type_matches(data, self.inputs):
      raise TypeError('''
        {} cannot encode type '{}' acceptable types are ({})
        '''.strip().format(
          self.name,
          type(data).__name__,
          ','.join(["'{}'".format(x.__name__) for x in self.inputs])
          ))
    return self._encode(data)

  def decode(self, data):
    if not self.type_matches(data, [self.outputs]):
      raise TypeError('''
        {} cannot decode type '{}' acceptable type is '{}'
        '''.strip().format(
          self.name,
          type(data).__name__,
          self.outputs.__name__
          ))
    return self._decode(data)


class InvertedEncoder(Encoder):

  def __init__(self, encoder):
    self.encoder = encoder
    self.file_suffixes = []
    for suffix in encoder.file_suffixes:
      self.file_suffixes.append('invert' + suffix)
    self.inputs = [encoder.outputs]
    self.outputs = encoder.inputs[0]

  def encode(self, data):
    return self.encoder.decode(data)

  def decode(self, data):
    return self.encoder.encode(data)



import lz4.frame
import six

from .encoder import Encoder

class Lz4Encoder(Encoder):
  file_suffixes = ['lz4']
  inputs = [six.binary_type]
  outputs = six.binary_type

  def __init__(self):
    super(Lz4Encoder, self)

  def _encode(self, data):
    return lz4.frame.compress(data)

  def _decode(self, data):
    return lz4.frame.decompress(data)



import gzip
from io import BytesIO
import six

from .encoder import Encoder

class GzipEncoder(Encoder):
  file_suffixes = ['gz', 'gzip']
  inputs = [six.binary_type]
  outputs = six.binary_type

  def __init__(self):
    super(GzipEncoder, self)

  def _encode(self, data):
    fgz = BytesIO()
    gzip_obj = gzip.GzipFile(mode='wb', fileobj=fgz)
    gzip_obj.write(data)
    gzip_obj.close()
    return fgz.getvalue()

  def _decode(self, data):
    fgz = BytesIO(data)
    gzip_obj = gzip.GzipFile(mode='r', fileobj=fgz)
    return gzip_obj.read()



import six

from .encoder import Encoder

class TextEncoder(Encoder):
  file_suffixes = ['txt']
  inputs = six.string_types
  outputs = six.binary_type

  def __init__(self, encoding='utf-8'):
    self.encoding = encoding
    #self.file_suffixes = [encoding.replace('-', '')]
    super(TextEncoder, self)

  def _encode(self, data):
    return data.encode(self.encoding)

  def _decode(self, data):
    return data.decode(self.encoding)



import unittest
import six

from generic_encoders import avro_encoder, AvroEncoder

class TestAvroEncoder(unittest.TestCase):

    def setUp(self):
        self.schema = {
            'doc': 'A weather reading.',
            'name': 'Weather',
            'namespace': 'test',
            'type': 'record',
            'fields': [
                {'name': 'station', 'type': 'string'},
                {'name': 'time', 'type': 'long'},
                {'name': 'temp', 'type': 'int'},
            ],
        }
        self.records = [
            {u'station': u'011990-99999', u'temp': 0, u'time': 1433269388},
            {u'station': u'011990-99999', u'temp': 22, u'time': 1433270389},
            {u'station': u'011990-99999', u'temp': -11, u'time': 1433273379},
            {u'station': u'012650-99999', u'temp': 111, u'time': 1433275478},
        ]


    def test_avro_encode_decode(self):
        encoder = AvroEncoder(self.schema)
        data = encoder.encode(self.records)
        self.assertEqual(self.records, list(encoder.decode(data)))

    def test_avro_encode_decode_with_snappy(self):
        encoder = AvroEncoder(self.schema, codec='snappy')
        data = encoder.encode(self.records)
        self.assertEqual(self.records, list(encoder.decode(data)))

    def test_avro_encode_decode_with_deflate(self):
        encoder = AvroEncoder(self.schema, codec='deflate')
        data = encoder.encode(self.records)
        self.assertEqual(self.records, list(encoder.decode(data)))

    def test_avro_decoder_doesnt_need_schema(self):
        encoder = AvroEncoder(self.schema)
        data = encoder.encode(self.records)
        self.assertEqual(self.records, list(avro_encoder.decode(data)))


if __name__ == '__main__':
    unittest.main()


from generic_encoders.encoders.gzip_encoder import Encoder
from generic_encoders.encoders.gzip_encoder import GzipEncoder
from generic_encoders.encoders.base64_encoder import Base64Encoder, UrlBase64Encoder
from generic_encoders.encoders.lz4_encoder import Lz4Encoder
from generic_encoders.encoders.json_encoder import JsonEncoder
from generic_encoders.encoders.dill_encoder import DillEncoder
from generic_encoders.encoders.msgpack_encoder import MsgPackEncoder
from generic_encoders.encoders.text_encoder import TextEncoder
from generic_encoders.encoders.csv_encoder import CsvEncoder
from generic_encoders.encoders.composed_encoder import ComposedEncoder
from generic_encoders.encoders.avro_encoder import AvroEncoder
from generic_encoders.encoders.snappy_encoder import SnappyEncoder


base64_encoder = Base64Encoder()
url_base64_encoder = UrlBase64Encoder()
gzip_encoder = GzipEncoder()
lz4_encoder = Lz4Encoder()
json_encoder = JsonEncoder()
dill_encoder = DillEncoder()
msgpack_encoder = MsgPackEncoder()
csv_encoder = CsvEncoder()
avro_encoder = AvroEncoder()
snappy_encoder = SnappyEncoder()

text_utf8_encoder = TextEncoder(encoding='utf-8')
text_ascii_encoder = TextEncoder(encoding='ascii')
text_latin_1_encoder = TextEncoder(encoding='latin-1')

Encoder.add_encoder(base64_encoder)
Encoder.add_encoder(gzip_encoder)
Encoder.add_encoder(lz4_encoder)
Encoder.add_encoder(json_encoder)
Encoder.add_encoder(dill_encoder)
Encoder.add_encoder(msgpack_encoder)
Encoder.add_encoder(text_utf8_encoder)
Encoder.add_encoder(text_ascii_encoder)
Encoder.add_encoder(text_latin_1_encoder)
Encoder.add_encoder(csv_encoder)
Encoder.add_encoder(avro_encoder)
Encoder.add_encoder(snappy_encoder)



import unittest
import six

from generic_encoders import *
from generic_encoders.encoders.composed_encoder import EncoderLinkError

class TestComposedEncoder(unittest.TestCase):

    def test_ascii_gzip_base64_encode_decode(self):
      encoder = ComposedEncoder(text_ascii_encoder, gzip_encoder, base64_encoder)
      self.assertEqual('abcdefg', encoder.decode(encoder.encode('abcdefg')))

    def test_ascii_gzip_base64_ascii_encode(self):
      encoder = ComposedEncoder(
        text_ascii_encoder,
        gzip_encoder,
        base64_encoder,
        text_ascii_encoder.inverted)
      self.assertEqual('abcdefg', encoder.decode(encoder.encode('abcdefg')))

    def test_ascii_msgpack_base64_ascii_encode(self):
      encoder = ComposedEncoder(
        text_ascii_encoder,
        msgpack_encoder,
        base64_encoder,
        text_ascii_encoder.inverted)
      self.assertEqual('abcdefg', encoder.decode(encoder.encode('abcdefg')))

    def test_connecting_incompatible_encoders_fails(self):
        if not six.PY2:
          with self.assertRaises(EncoderLinkError) as context:
            encoder = ComposedEncoder(base64_encoder, text_ascii_encoder)

if __name__ == '__main__':
    unittest.main()


import os
from setuptools import setup, find_packages

dir_path = os.path.dirname(os.path.realpath(__file__))

VERSION = open(os.path.join(dir_path, 'VERSION')).read()

snappy_version = "0.5.3"
avro_version = "0.21.7"

setup(
  name = 'generic-encoders',
  packages = find_packages(),
  version = VERSION,
  description = '''
  A set of encoders which provide a simple string/byte based interface.
  ''',
  long_description=open(os.path.join(dir_path, 'README.md')).read(),
  long_description_content_type='text/markdown',
  author = 'Marco Montagna',
  author_email = 'marcojoemontagna@gmail.com',
  url = 'https://github.com/mmontagna/generic-encoders',
  keywords = ['encoders', 'gzip', 'compression', 'decompression'],
  classifiers=(
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'Natural Language :: English',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python',
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.6',
  ),
  data_files = [('', ['LICENSE', 'VERSION', 'README.md'])],
  include_package_data=True,
  python_requires=">=2.7",
  license=open(os.path.join(dir_path, 'LICENSE')).read(),
  install_requires=[
    "six>=1.9.0",
    "lz4~=1.0.0",
    "dill~=0.2.5",
    "msgpack~=0.5.6",
    "simplejson>=3.8.0",
  ],
  extras_require={
      'avro': ["fastavro~={}".format(avro_version)],
      'snappy': ["python-snappy~={}".format(snappy_version)],
      'dev': [
        "fastavro~={}".format(avro_version),
        "python-snappy~={}".format(snappy_version)
      ],
      'all':  [
        "fastavro~={}".format(avro_version),
        "python-snappy~={}".format(snappy_version)
      ],
  },
  entry_points = {
  },
)


import gc
import struct
import datetime

import msgpack
import six

from .encoder import Encoder

class MsgPackEncoder(Encoder):
  file_suffixes = ['msgpack', 'msgpk']
  inputs = [object]
  outputs = six.binary_type

  def __init__(self, use_list=False, raw=False, use_bin_type=False):
    """
    use_list = False converts tuples to lists for a speed improvement on unpacking.
    """
    self.use_list = use_list
    self.raw = raw
    self.use_bin_type = use_bin_type
    super(MsgPackEncoder, self)

  def _encode(self, data):
    return msgpack.packb(
      data,
      use_bin_type=True,
      default=self._default_packer
      )

  def _decode(self, data):
    try:
      if len(data) > 1000:
        gc.disable()
      return msgpack.unpackb(
        data,
        ext_hook=self._ext_hook,
        use_list=self.use_list,
        raw=self.raw
        )
    finally:
      if len(data) > 1000:
        gc.enable()

  def _default_packer(self, obj):
    if isinstance(obj, datetime.datetime):
      return msgpack.ExtType(100, struct.pack('L', int(obj.strftime("%s"))))
    elif isinstance(obj, datetime.date):
      return msgpack.ExtType(101, struct.pack('L', int(obj.strftime("%s"))))
    raise TypeError("Unknown type: {}".format(obj))

  def _ext_hook(self, code, data):
    if code == 100:
        return datetime.datetime.utcfromtimestamp(struct.unpack('L', data)[0])
    elif code == 101:
        return datetime.date.fromtimestamp(struct.unpack('L', data)[0])
    return msgpack.ExtType(code, data)

    use_bin_type=self.use_bin_type